@@ -28,9 +28,11 @@ def statements_for_changes(
     things_target,
     creations_only=False,
     drops_only=False,
+    modifications_only=False,
     modifications=True,
     dependency_ordering=False,
     add_dependents_for_modified=False,
+    modifications_as_alters=False,
 ):
     added, removed, modified, unmodified = differences(things_from, things_target)
 
@@ -41,9 +43,11 @@ def statements_for_changes(
         replaceable=None,
         creations_only=creations_only,
         drops_only=drops_only,
+        modifications_only=modifications_only,
         modifications=modifications,
         dependency_ordering=dependency_ordering,
         old=things_from,
+        modifications_as_alters=modifications_as_alters,
     )
 
 
@@ -57,21 +61,40 @@ def statements_from_differences(
     modifications=True,
     dependency_ordering=False,
     old=None,
+    modifications_only=False,
+    modifications_as_alters=False,
 ):
     replaceable = replaceable or set()
     statements = Statements()
-    if not creations_only:
-        pending_drops = set(removed)
-        if modifications:
+
+    pending_creations = set()
+    pending_drops = set()
+
+    creations = not (drops_only or modifications_only)
+    drops = not (creations_only or modifications_only)
+    modifications = (
+        modifications or modifications_only and not (creations_only or drops_only)
+    )
+
+    drop_and_recreate = modifications and not modifications_as_alters
+    alters = modifications and modifications_as_alters
+
+    if drops:
+        pending_drops |= set(removed)
+
+    if creations:
+        pending_creations |= set(added)
+
+    if drop_and_recreate:
+        if drops:
             pending_drops |= set(modified) - replaceable
-    else:
-        pending_drops = set()
-    if not drops_only:
-        pending_creations = set(added)
-        if modifications:
+
+        if creations:
             pending_creations |= set(modified)
-    else:
-        pending_creations = set()
+
+    if alters:
+        for k, v in modified.items():
+            statements += v.alter_statements(old[k])
 
     def has_remaining_dependents(v, pending_drops):
         if not dependency_ordering:
@@ -87,13 +110,13 @@ def statements_from_differences(
 
     while True:
         before = pending_drops | pending_creations
-        if not creations_only:
+        if drops:
             for k, v in removed.items():
                 if not has_remaining_dependents(v, pending_drops):
                     if k in pending_drops:
                         statements.append(old[k].drop_statement)
                         pending_drops.remove(k)
-        if not drops_only:
+        if creations:
             for k, v in added.items():
                 if not has_uncreated_dependencies(v, pending_creations):
                     if k in pending_creations:
@@ -101,12 +124,12 @@ def statements_from_differences(
                         pending_creations.remove(k)
         if modifications:
             for k, v in modified.items():
-                if not creations_only:
+                if drops:
                     if not has_remaining_dependents(v, pending_drops):
                         if k in pending_drops:
                             statements.append(old[k].drop_statement)
                             pending_drops.remove(k)
-                if not drops_only:
+                if creations:
                     if not has_uncreated_dependencies(v, pending_creations):
                         if k in pending_creations:
                             statements.append(v.create_statement)
@@ -123,7 +146,9 @@ def statements_from_differences(
     return statements
 
 
-def get_enum_modifications(tables_from, tables_target, enums_from, enums_target):
+def get_enum_modifications(
+    tables_from, tables_target, enums_from, enums_target, return_tuple=False
+):
     _, _, e_modified, _ = differences(enums_from, enums_target)
     _, _, t_modified, _ = differences(tables_from, tables_target)
     pre = Statements()
@@ -167,7 +192,11 @@ def get_enum_modifications(tables_from, tables_target, enums_from, enums_target)
         drop_statement = e.drop_statement_with_rename(unwanted_name)
 
         post.append(drop_statement)
-    return pre + recreate + post
+
+    if return_tuple:
+        return pre, recreate + post
+    else:
+        return pre + recreate + post
 
 
 def get_table_changes(
@@ -183,14 +212,20 @@ def get_table_changes(
     statements = Statements()
     for t, v in removed.items():
         statements.append(v.drop_statement)
+
+    enums_pre, enums_post = get_enum_modifications(
+        tables_from, tables_target, enums_from, enums_target, return_tuple=True
+    )
+
+    statements += enums_pre
+
     for t, v in added.items():
         statements.append(v.create_statement)
         if v.rowsecurity:
             rls_alter = v.alter_rls_statement
             statements.append(rls_alter)
-    statements += get_enum_modifications(
-        tables_from, tables_target, enums_from, enums_target
-    )
+
+    statements += enums_post
 
     for t, v in modified.items():
         before = tables_from[t]
@@ -208,7 +243,13 @@ def get_table_changes(
         if v.parent_table != before.parent_table:
             statements += v.attach_detach_statements(before)
 
-    for t, v in modified.items():
+    modified_order = list(modified.keys())
+
+    modified_order.sort(key=lambda x: modified[x].is_inheritance_child_table)
+
+    for t in modified_order:
+        v = modified[t]
+
         before = tables_from[t]
 
         if not v.is_alterable:
@@ -222,10 +263,19 @@ def get_table_changes(
 
             # there's no way to alter a table into/out of generated state
             # so you gotta drop/recreate
-            if c.is_generated != c_before.is_generated:
+
+            generated_status_changed = c.is_generated != c_before.is_generated
+
+            inheritance_status_changed = c.is_inherited != c_before.is_inherited
+
+            if inheritance_status_changed or generated_status_changed:
                 del c_modified[k]
-                c_added[k] = c
-                c_removed[k] = c_before
+
+                if not c_before.is_inherited:
+                    c_removed[k] = c_before
+
+                if not c.is_inherited:
+                    c_added[k] = c
 
         for k, c in c_removed.items():
             alter = v.alter_table_statement(c.drop_column_clause)
@@ -234,7 +284,8 @@ def get_table_changes(
             alter = v.alter_table_statement(c.add_column_clause)
             statements.append(alter)
         for k, c in c_modified.items():
-            statements += c.alter_table_statements(before.columns[k], t)
+            c_before = before.columns[k]
+            statements += c.alter_table_statements(c_before, t)
 
         if v.rowsecurity != before.rowsecurity:
             rls_alter = v.alter_rls_statement
@@ -280,6 +331,8 @@ def get_selectable_differences(
         other_from, other_target
     )
 
+    _, _, modified_enums, _ = differences(enums_from, enums_target)
+
     changed_all = {}
     changed_all.update(modified_tables)
     changed_all.update(modified_other)
@@ -291,12 +344,16 @@ def get_selectable_differences(
     not_replaceable = set()
 
     if add_dependents_for_modified:
+
         for k, m in changed_all.items():
             old = selectables_from[k]
 
             if k in modified_all and m.can_replace(old):
                 if not m.is_table:
-                    replaceable.add(k)
+                    changed_enums = [_ for _ in m.dependent_on if _ in modified_enums]
+                    if not changed_enums:
+                        replaceable.add(k)
+
                 continue
 
             for d in m.dependents_all:
@@ -375,6 +432,10 @@ def get_selectable_changes(
     sequences_from,
     sequences_target,
     add_dependents_for_modified=True,
+    tables_only=False,
+    non_tables_only=False,
+    drops_only=False,
+    creations_only=False,
 ):
     (
         tables_from,
@@ -398,91 +459,160 @@ def get_selectable_changes(
     def functions(d):
         return {k: v for k, v in d.items() if v.relationtype == "f"}
 
-    statements += statements_from_differences(
-        added_other,
-        removed_other,
-        modified_other,
-        replaceable=replaceable,
-        drops_only=True,
-        dependency_ordering=True,
-        old=selectables_from,
-    )
+    if not tables_only:
+        if not creations_only:
+            statements += statements_from_differences(
+                added_other,
+                removed_other,
+                modified_other,
+                replaceable=replaceable,
+                drops_only=True,
+                dependency_ordering=True,
+                old=selectables_from,
+            )
 
-    statements += get_table_changes(
-        tables_from,
-        tables_target,
-        enums_from,
-        enums_target,
-        sequences_from,
-        sequences_target,
-    )
+    if not non_tables_only:
+        statements += get_table_changes(
+            tables_from,
+            tables_target,
+            enums_from,
+            enums_target,
+            sequences_from,
+            sequences_target,
+        )
 
-    if any([functions(added_other), functions(modified_other)]):
-        statements += ["set check_function_bodies = off;"]
+    if not tables_only:
+        if not drops_only:
+            if any([functions(added_other), functions(modified_other)]):
+                statements += ["set check_function_bodies = off;"]
 
-    statements += statements_from_differences(
-        added_other,
-        removed_other,
-        modified_other,
-        replaceable=replaceable,
-        creations_only=True,
-        dependency_ordering=True,
-        old=selectables_from,
-    )
+            statements += statements_from_differences(
+                added_other,
+                removed_other,
+                modified_other,
+                replaceable=replaceable,
+                creations_only=True,
+                dependency_ordering=True,
+                old=selectables_from,
+            )
     return statements
 
 
 class Changes(object):
-    def __init__(self, i_from, i_target):
+    def __init__(self, i_from, i_target, ignore_extension_versions=False):
         self.i_from = i_from
         self.i_target = i_target
+        self.ignore_extension_versions = ignore_extension_versions
 
-    def __getattr__(self, name):
-        if name == "non_pk_constraints":
-            a = self.i_from.constraints.items()
-            b = self.i_target.constraints.items()
-            a_od = od((k, v) for k, v in a if v.constraint_type != PK)
-            b_od = od((k, v) for k, v in b if v.constraint_type != PK)
-            return partial(statements_for_changes, a_od, b_od)
-
-        elif name == "pk_constraints":
-            a = self.i_from.constraints.items()
-            b = self.i_target.constraints.items()
-            a_od = od((k, v) for k, v in a if v.constraint_type == PK)
-            b_od = od((k, v) for k, v in b if v.constraint_type == PK)
-            return partial(statements_for_changes, a_od, b_od)
-
-        elif name == "selectables":
-            return partial(
-                get_selectable_changes,
-                od(sorted(self.i_from.selectables.items())),
-                od(sorted(self.i_target.selectables.items())),
-                self.i_from.enums,
-                self.i_target.enums,
-                self.i_from.sequences,
-                self.i_target.sequences,
-            )
-
-        elif name == "triggers":
-            return partial(
-                get_trigger_changes,
-                od(sorted(self.i_from.triggers.items())),
-                od(sorted(self.i_target.triggers.items())),
-                od(sorted(self.i_from.selectables.items())),
-                od(sorted(self.i_target.selectables.items())),
-                self.i_from.enums,
-                self.i_target.enums,
-            )
-
-        elif name == "sequences":
+    @property
+    def extensions(self):
+        if self.ignore_extension_versions:
             return partial(
                 statements_for_changes,
-                getattr(self.i_from, name),
-                getattr(self.i_target, name),
+                self.i_from.extensions,
+                self.i_target.extensions,
                 modifications=False,
             )
+        else:
+            return partial(
+                statements_for_changes,
+                self.i_from.extensions,
+                self.i_target.extensions,
+                modifications_as_alters=True,
+            )
 
-        elif name in THINGS:
+    @property
+    def selectables(self):
+        return partial(
+            get_selectable_changes,
+            od(sorted(self.i_from.selectables.items())),
+            od(sorted(self.i_target.selectables.items())),
+            self.i_from.enums,
+            self.i_target.enums,
+            self.i_from.sequences,
+            self.i_target.sequences,
+        )
+
+    @property
+    def tables_only_selectables(self):
+        return partial(
+            get_selectable_changes,
+            od(sorted(self.i_from.selectables.items())),
+            od(sorted(self.i_target.selectables.items())),
+            self.i_from.enums,
+            self.i_target.enums,
+            self.i_from.sequences,
+            self.i_target.sequences,
+            tables_only=True,
+        )
+
+    @property
+    def non_table_selectable_drops(self):
+        return partial(
+            get_selectable_changes,
+            od(sorted(self.i_from.selectables.items())),
+            od(sorted(self.i_target.selectables.items())),
+            self.i_from.enums,
+            self.i_target.enums,
+            self.i_from.sequences,
+            self.i_target.sequences,
+            drops_only=True,
+            non_tables_only=True,
+        )
+
+    @property
+    def non_table_selectable_creations(self):
+        return partial(
+            get_selectable_changes,
+            od(sorted(self.i_from.selectables.items())),
+            od(sorted(self.i_target.selectables.items())),
+            self.i_from.enums,
+            self.i_target.enums,
+            self.i_from.sequences,
+            self.i_target.sequences,
+            creations_only=True,
+            non_tables_only=True,
+        )
+
+    @property
+    def non_pk_constraints(self):
+        a = self.i_from.constraints.items()
+        b = self.i_target.constraints.items()
+        a_od = od((k, v) for k, v in a if v.constraint_type != PK)
+        b_od = od((k, v) for k, v in b if v.constraint_type != PK)
+        return partial(statements_for_changes, a_od, b_od)
+
+    @property
+    def pk_constraints(self):
+        a = self.i_from.constraints.items()
+        b = self.i_target.constraints.items()
+        a_od = od((k, v) for k, v in a if v.constraint_type == PK)
+        b_od = od((k, v) for k, v in b if v.constraint_type == PK)
+        return partial(statements_for_changes, a_od, b_od)
+
+    @property
+    def triggers(self):
+        return partial(
+            get_trigger_changes,
+            od(sorted(self.i_from.triggers.items())),
+            od(sorted(self.i_target.triggers.items())),
+            od(sorted(self.i_from.selectables.items())),
+            od(sorted(self.i_target.selectables.items())),
+            self.i_from.enums,
+            self.i_target.enums,
+        )
+
+    @property
+    def sequences(self):
+        return partial(
+            statements_for_changes,
+            self.i_from.sequences,
+            self.i_target.sequences,
+            modifications=False,
+        )
+
+    def __getattr__(self, name):
+        if name in THINGS:
             return partial(
                 statements_for_changes,
                 getattr(self.i_from, name),
