@@ -12,28 +12,39 @@ class Migration(object):
     The main class of migra
     """
 
-    def __init__(self, x_from, x_target, schema=None):
+    def __init__(self, x_from, x_target, schema=None, exclude_schema=None):
         self.statements = Statements()
         self.changes = Changes(None, None)
+        if schema and exclude_schema:
+            raise ValueError("You cannot have both a schema and excluded schema")
         self.schema = schema
+        self.exclude_schema = exclude_schema
         if isinstance(x_from, DBInspector):
             self.changes.i_from = x_from
         else:
-            self.changes.i_from = get_inspector(x_from, schema=schema)
+            self.changes.i_from = get_inspector(
+                x_from, schema=schema, exclude_schema=exclude_schema
+            )
             if x_from:
                 self.s_from = x_from
         if isinstance(x_target, DBInspector):
             self.changes.i_target = x_target
         else:
-            self.changes.i_target = get_inspector(x_target, schema=schema)
+            self.changes.i_target = get_inspector(
+                x_target, schema=schema, exclude_schema=exclude_schema
+            )
             if x_target:
                 self.s_target = x_target
 
     def inspect_from(self):
-        self.changes.i_from = get_inspector(self.s_from, schema=self.schema)
+        self.changes.i_from = get_inspector(
+            self.s_from, schema=self.schema, exclude_schema=self.exclude_schema
+        )
 
     def inspect_target(self):
-        self.changes.i_target = get_inspector(self.s_target, schema=self.schema)
+        self.changes.i_target = get_inspector(
+            self.s_target, schema=self.schema, exclude_schema=self.exclude_schema
+        )
 
     def clear(self):
         self.statements = Statements()
@@ -41,7 +52,9 @@ class Migration(object):
     def apply(self):
         for stmt in self.statements:
             raw_execute(self.s_from, stmt)
-        self.changes.i_from = get_inspector(self.s_from, schema=self.schema)
+        self.changes.i_from = get_inspector(
+            self.s_from, schema=self.schema, exclude_schema=self.exclude_schema
+        )
         safety_on = self.statements.safe
         self.clear()
         self.set_safety(safety_on)
@@ -74,10 +87,13 @@ class Migration(object):
         if privileges:
             self.add(self.changes.privileges(drops_only=True))
         self.add(self.changes.non_pk_constraints(drops_only=True))
+
+        self.add(self.changes.non_table_selectable_drops())
+
         self.add(self.changes.pk_constraints(drops_only=True))
         self.add(self.changes.indexes(drops_only=True))
 
-        self.add(self.changes.selectables())
+        self.add(self.changes.tables_only_selectables())
 
         self.add(self.changes.sequences(drops_only=True))
         self.add(self.changes.enums(drops_only=True, modifications=False))
@@ -85,6 +101,9 @@ class Migration(object):
         self.add(self.changes.indexes(creations_only=True))
         self.add(self.changes.pk_constraints(creations_only=True))
         self.add(self.changes.non_pk_constraints(creations_only=True))
+
+        self.add(self.changes.non_table_selectable_creations())
+
         if privileges:
             self.add(self.changes.privileges(creations_only=True))
         self.add(self.changes.rlspolicies(creations_only=True))
